import json
import os
import requests
import time

from logs import Log
from prometheus_client.core import GaugeMetricFamily, REGISTRY
from prometheus_client import start_http_server


class PBPCollector:
    def __init__(self):
        self.log = Log("PBPCollector")
        self.partants = GaugeMetricFamily(
            "pbp_partants", "Participant in PBP", labels=["partant_num"]
        )
        self.maxpartants = GaugeMetricFamily(
            "pbp_max_partants", "Max participant in PBP", labels=["partant_num"]
        )

    def collect(self):
        yield self.partants
        yield self.maxpartants

    def fetch(self):
        self.log.info("Fetch data for depart 1")
        self.fetcher(depart=1, special=False)
        self.log.info("Fetch data for depart 2")
        self.fetcher(depart=2, special=False)
        self.log.info("Fetch data for depart 3")
        self.fetcher(depart=3, special=False)
        self.log.info("Fetch data for special depart 2")
        self.fetcher(depart=2, special=True)
        self.log.info("Fetch data for special depart 3")
        self.fetcher(depart=3, special=True)

    def fetcher(self, depart: int = 1, special: bool = False):
        pbp_query = self.query(depart, special)
        if not pbp_query:
            return False
        self.log.info(pbp_query)
        for depart in pbp_query["departs"].keys():
            for key in pbp_query["departs"][depart].keys():
                if special:
                    depart_name = f"special_{depart}"
                else:
                    depart_name = f"{depart}"
                if key == "partants":
                    self.partants.add_metric(
                        [depart_name], pbp_query["departs"][depart][key]
                    )
                elif key == "max":
                    self.maxpartants.add_metric(
                        [depart_name], pbp_query["departs"][depart][key]
                    )

    def query(self, depart: int = 1, special: bool = False):
        url = "https://inscription.paris-brest-paris.org/controleur/pbp/json_pbp.php"
        payload = {"search": "Depart", "special": special, "depart": f"{str(depart)}"}
        try:
            r = requests.post(url, json=payload)
            r.raise_for_status()
            return r.json()
        except requests.JSONDecodeError as err:
            self.log.error("Failed to decode JSON")
            return False
        except requests.exceptions.HTTPError as err:
            self.log.error(
                f"Encounter error {err.response.status_code} when try to get data"
            )
            return False
        except requests.exceptions.RequestException as e:
            self.log.error("An error occured")
            return False



def main():
    log = Log("main")

    polling_interval = int(os.getenv("POLLING_INTERVAL", "5"))
    exporter_port = int(os.getenv("EXPORTER_PORT", "9781"))

    log.info(f"Polling interval is set to {polling_interval}")
    log.info(f"HTTP listen port is set to {exporter_port}")

    collector = PBPCollector()
    REGISTRY.register(collector)
    log.info("Starting server")
    start_http_server(exporter_port)
    log.info("Server started")
    while True:
        collector.fetch()
        time.sleep(polling_interval)


if __name__ == "__main__":
    main()
